--
-- MBgearbox
-- 19/07/09
-- MBgearbox for combine
-- guimchevalier from TSF3
-- Knagsted from ModSource

MBgearbox = {};

function MBgearbox.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Motorized, specializations);
end;

function MBgearbox:load(xmlFile)
	-- gears
    self.numGear = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.gears#count"), 0);
	self.gears = {};
    for i=1, self.numGear do
        local geari = string.format("vehicle.gears.gear%d", i);
        self.gears[i] = getXMLFloat(xmlFile, geari .. "#value");
    end;
	self.miniGear = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.gears#start"), 1);
	self.gearNum = self.miniGear;
	-- hydro
	self.hydro = 0;
	self.oldInput = 1;
	--rpms
	self.rpms = {};
    local rpm1, rpm2, rpm3 = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rpms#motor"));
	self.rpms[1] = {};
    self.rpms[1] = {rpm1, rpm2, rpm3};
    local rpm1, rpm2, rpm3 = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rpms#disp"));
    self.rpms[2] = {};
	self.rpms[2] = {rpm1, rpm2, rpm3};
	self.rpmNum = 1;
	-- real motor sound
	local motorMBSound = getXMLString(xmlFile, "vehicle.motorMBSound#file");
    if motorMBSound ~= nil and motorMBSound ~= "" then
        motorMBSound = Utils.getFilename(motorMBSound, self.baseDirectory);
        self.motorMBSound = createSample("motorMBSound");
        loadSample(self.motorMBSound, motorMBSound, false);
        self.motorMBSoundPitchOffset = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.motorMBSound#pitchOffset"), 0);
        self.motorMBSoundVolume = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.motorMBSound#volume"), 1.0);
		self.motorMBSoundPitchScale = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.motorMBSound#pitchScale"), 0.05);
		self.motorMBSoundPitchMax = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.motorMBSound#pitchMax"), 2.0);
    end;
	
	self.wheel1 = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "vehicle.wheel1#index"));
    self.wheel2 = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "vehicle.wheel2#index"));
	
	self.hydroActivated = true;
	
	local motorMaxRpmStr = getXMLString(xmlFile, "vehicle.motor#maxRpm");
    local motorMaxRpm1, motorMaxRpm2, motorMaxRpm3 = Utils.getVectorFromString(motorMaxRpmStr);
    motorMaxRpm1 = Utils.getNoNil(motorMaxRpm1, 800);
    motorMaxRpm2 = Utils.getNoNil(motorMaxRpm2, 1000);
    motorMaxRpm3 = Utils.getNoNil(motorMaxRpm3, 1800);
    local motorMaxRpm = {motorMaxRpm1, motorMaxRpm2, motorMaxRpm3};
    self.motorMaxRpmLimit = motorMaxRpm;
	
	self.lastRPMLimitSpeed3 = motorMaxRpm3;
end;

function MBgearbox:delete()
    if self.motorMBSound ~= nil then
        delete(self.motorMBSound);
    end;
end;

function MBgearbox:mouseEvent(posX, posY, isDown, isUp, button)
end;

function MBgearbox:stopMotor()
    if self.motorMBSound ~= nil then
        stopSample(self.motorMBSound);
    end;
	self.hydro = 0;
end;

function MBgearbox:keyEvent(unicode, sym, modifier, isDown)
    if self:getIsActiveForInput() and self.isEntered then
	    if InputBinding.isPressed(InputBinding.JDTSPEEDUP) and self.gearNum < self.numGear then
			self.gearNum = self.gearNum + 1;
		elseif InputBinding.isPressed(InputBinding.JDTSPEEDDOWN) and self.gearNum > 1 then
			self.gearNum = self.gearNum - 1;
		end;
		
		if InputBinding.isPressed(InputBinding.JDTRPMUP) then
			if self.hydroActivated and self.rpmNum < 3 then
				self.rpmNum = self.rpmNum + 1;
			end;
		elseif InputBinding.isPressed(InputBinding.JDTRPMDOWN) then
			if self.hydroActivated and self.rpmNum > 1 then
				self.rpmNum = self.rpmNum - 1;
			end;
		end;
		
		if InputBinding.isPressed(InputBinding.JDTHYDROOFF) then
			self.hydroActivated = not self.hydroActivated;
		end;
	end;
end;

function MBgearbox:startMotor()
	self.playMotorMBSound = true;
	self.oldInput = 1;
end;

function MBgearbox:update(dt)             	
	if self.isEntered and self.isMotorStarted then
		-- hydro
		local input = InputBinding.getAnalogInputAxis(InputBinding.AXIS_FORWARD);
		if InputBinding.isAxisZero(input) then
			input = InputBinding.getDigitalInputAxis(InputBinding.AXIS_FORWARD);
		end;
		if input==0 then
			if InputBinding.isPressed(InputBinding.JDTHYDROUP) then
				input = -1;
			elseif InputBinding.isPressed(InputBinding.JDTHYDRODOWN) then
				input = 1;
			end;
		end;
		
		if InputBinding.isPressed(InputBinding.JDTRPMUP) then
			if not self.hydroActivated then
				if self.motor.speedLevel ~= 0 then
					if self.motor.maxRpm[self.motor.speedLevel] <= (self.motorMaxRpmLimit[3] - 10) then
						self.motor.maxRpm[self.motor.speedLevel] = self.motor.maxRpm[self.motor.speedLevel] + 10;
					end;
				else
					if self.motor.maxRpm[3] <= (self.motorMaxRpmLimit[3] - 10) then
						self.motor.maxRpm[3] = self.motor.maxRpm[3] + 10;
						self.lastRPMLimitSpeed3 = self.motor.maxRpm[3];
					end;
				end;
			end;
		elseif InputBinding.isPressed(InputBinding.JDTRPMDOWN) then
			if not self.hydroActivated then
				if self.motor.speedLevel ~= 0 then
					if self.motor.maxRpm[self.motor.speedLevel] >= 10 then
						self.motor.maxRpm[self.motor.speedLevel] = self.motor.maxRpm[self.motor.speedLevel] - 10;
					end;
				else
					if self.motor.maxRpm[3] >= 10 then
						self.motor.maxRpm[3] = self.motor.maxRpm[3] - 10;
						self.lastRPMLimitSpeed3 = self.motor.maxRpm[3];
					end;
				end
			end;
		end;
		
		if self.hydro == 0 and self.oldInput == 0 and input ~= 0 then
			if input < 0 then
				self.hydro = 5;
			else
				self.hydro = -5;
			end;
		elseif self.hydro ~= 0 then
			if (self.hydro > 0 and input > 0) or (self.hydro < 0 and input < 0) then
				self.hydro = self.hydro - 4*input;
				if self.hydro < 10 and self.hydro > -10 then
					self.hydro = 0;
				end;
			elseif self.hydro > -100 and self.hydro < 100 then
				self.hydro = self.hydro - 0.5*input;
			end;
		end;
		self.oldInput = input;
		
		if self.motor.speedLevel ~= 0 then
			self.hydro = 0;
			self.motor.maxRpm[3] = self.lastRPMLimitSpeed3;
		end;
	
		if self.hydroActivated then
			local newRpm = math.abs(self.hydro)*self.gears[self.gearNum]/100 * (self.rpms[2][self.rpmNum]/self.rpms[2][3]);
			self.motor.maxRpm[3] = newRpm;
		
			if self.hydro ~= 0 then
				if self.firstTimeRun and self.steeringEnabled then
					local acceleration = self.hydro/math.abs(self.hydro);
					if newRpm < 250 then
						acceleration = acceleration * 0.4;
					elseif newRpm < 750 and newRpm >= 250 then
						acceleration = acceleration * 0.55;
					elseif newRpm < 1250 and newRpm >= 750 then
						acceleration = acceleration * 0.65;
					elseif newRpm < 1750 and newRpm >= 1250 then
						acceleration = acceleration * 0.8;
					elseif newRpm < 2250 and newRpm >= 1750 then
						acceleration = acceleration * 0.9;
					elseif newRpm >= 2250 then
						acceleration = acceleration * 1;
					end;
					WheelsUtil.updateWheelsPhysics(self, dt, self.lastSpeed, acceleration, false, 1);
				end;
			end;
		else
			self.hydro = 0;
		end;
	end;
	
	  if self.wheel1 ~= nil then
		  local x, y, z = getRotation(self.wheels[3].repr);
		  setRotation(self.wheel1, x,0,0);
	  end;

	  if self.wheel2 ~= nil then
		  local x, y, z = getRotation(self.wheels[4].repr);
		  setRotation(self.wheel2, x,0,0);
	  end;
	  
	  
	  
	-- motorMBsound	
	if self.isMotorStarted and self.motorMBSound ~= nil and self.playMotorSoundTime <= self.time and self:getIsActiveForSound() then
		setSamplePitch(self.motorMBSound, math.min(self.motorMBSoundPitchOffset + self.motorMBSoundPitchScale*math.abs(self.rpms[1][self.rpmNum] / 60), self.motorMBSoundPitchMax));
		if self.playMotorMBSound then
			playSample(self.motorMBSound, 0, self.motorMBSoundVolume, 0);
			self.playMotorMBSound = false;
		end;
	end;

	if self.isEntered == nil or not self.isEntered then
	        stopSample(self.motorMBSound);
	end;
end;

function MBgearbox:draw()
		if self.HUDActive then
			if self.isMotorStarted then
				-- g_currentMission:addExtraPrintText(" last"..string.format("%d",self.lastRoundPerMinute));
				-- g_currentMission:addExtraPrintText(" last"..string.format("%d",self.oldInput *100));
				setTextBold(true);
				setTextColor(0, 0, 0, 1.0);
				if self.hydroActivated then
					renderText(0.909, 0.24, 0.0250, string.format("%4d ",self.rpms[2][self.rpmNum]).."rpm");
					if self.motor.speedLevel == 0 or self.motor.speedLevel == 3 then
						renderText(0.907, 0.27, 0.055,string.format("%d ",self.gearNum));
						local hydro = math.floor((self.hydro+2.5)/5)*5;
						renderText(0.932, 0.28, 0.0250, string.format("%d ",hydro).."%");
					end;
				else
					if self.motor.speedLevel ~= 0 then
						renderText(0.909, 0.24, 0.0250, string.format("%4d ",self.motor.maxRpm[self.motor.speedLevel]).."rpm");
					else
						renderText(0.909, 0.24, 0.0250, string.format("%4d ",self.motor.maxRpm[3]).."rpm");
					end;
				end;
			end;
		end;
		if self.hydroActivated then
			g_currentMission:addExtraPrintText("Touche Z: Deactivate hydrostatic gearbox");
		else
			g_currentMission:addExtraPrintText("Touche Z: Reeactivate hydrostatic gearbox");
		end;
end;
